import { createStore, createEvent, createEffect } from "effector";
import { User, IdTokenClaims } from "oidc-client-ts"; // UserManager removed
import { getUserManager } from "./oidc-config"; // Import getUserManager

// Define types
export interface AppUser {
  id: string;
  name: string;
  email: string;
  roles: string[];
}

interface OidcUserProfile {
  sub: string;
  name?: string;
  preferred_username?: string;
  email?: string;
  roles?: string[];
  [key: string]: unknown;
}

interface OidcUser extends Omit<User, "profile"> {
  profile: OidcUserProfile & Partial<IdTokenClaims>;
  [key: string]: unknown;
}

export interface AuthState {
  user: AppUser | null;
  isAuthenticated: boolean;
  isLoading: boolean;
  error: string | null;
  manuallyLoggedOut: boolean; // Track if user manually logged out
}

// Define events
export const setUser = createEvent<AppUser | null>();
export const setIsLoading = createEvent<boolean>();
export const setError = createEvent<string | null>();
export const logout = createEvent();

// Define effects
export const loginFx = createEffect(async () => {
  const userManager = getUserManager(); // Get instance

  // First try to get the current user
  const currentUser = await userManager.getUser();

  // If we have a valid user that's not expired, we're already logged in
  if (currentUser && !currentUser.expired) {
    console.log("loginFx: User is already logged in and token is valid");
    return currentUser;
  }

  // Try silent login first
  try {
    console.log("loginFx: Attempting silent login");
    const user = await userManager.signinSilent();
    console.log("loginFx: Silent login successful");
    return user;
  } catch (silentError) {
    console.log(
      "loginFx: Silent login failed, falling back to redirect:",
      silentError,
    );
    // Fall back to redirect login
    await userManager.signinRedirect();
    return null; // This line won't be reached as signinRedirect redirects the page
  }
});

export const logoutFx = createEffect(async () => {
  // Set the manuallyLoggedOut flag in localStorage first
  setManuallyLoggedOutInStorage(true);
  console.log("logoutFx: Set manuallyLoggedOut to true in localStorage");

  // Clear the user from the store before redirecting
  // This will prevent auto-login from triggering
  logout();

  // Get the UserManager instance
  const userManager = getUserManager();

  // Use a small delay to ensure the flag is set before redirecting
  await new Promise((resolve) => setTimeout(resolve, 100));

  // Now redirect to the identity provider for logout
  await userManager.signoutRedirect();
});

export const checkAuthFx = createEffect<void, OidcUser | null, Error>(
  async () => {
    console.log("checkAuthFx: Starting to check authentication status");
    try {
      const userManager = getUserManager(); // Get instance
      console.log("checkAuthFx: Got UserManager instance");

      // First try to get the user from storage
      let user = await userManager.getUser();
      console.log("checkAuthFx: User from storage:", user);

      // If we have a user but the token is expired or about to expire, try silent renew
      if (
        user &&
        (user.expired || (user.expires_in && user.expires_in < 300))
      ) {
        console.log(
          "checkAuthFx: Token is expired or about to expire, attempting silent renew",
        );
        try {
          user = await userManager.signinSilent();
          console.log("checkAuthFx: Silent renew successful, new user:", user);
        } catch (renewError) {
          console.warn("checkAuthFx: Silent renew failed:", renewError);
          // Continue with the expired user, the app will handle redirect if needed
        }
      }

      if (!user) {
        console.log("checkAuthFx: No user found, user is not authenticated");
      } else {
        console.log("checkAuthFx: User found, user is authenticated");
      }

      return user as OidcUser | null;
    } catch (error) {
      console.error("checkAuthFx: Error checking authentication:", error);
      throw error;
    }
  },
);

export const handleCallbackFx = createEffect<void, OidcUser, Error>(
  async () => {
    console.log("handleCallbackFx: Starting to process callback");
    try {
      const userManager = getUserManager(); // Get instance
      console.log("handleCallbackFx: Got UserManager instance");

      const user = await userManager.signinRedirectCallback();
      console.log(
        "handleCallbackFx: Successfully processed callback, user:",
        user,
      );

      return user as OidcUser;
    } catch (error) {
      console.error("handleCallbackFx: Error processing callback:", error);
      throw error;
    }
  },
);

// Helper functions for localStorage
const getManuallyLoggedOutFromStorage = (): boolean => {
  try {
    const value = localStorage.getItem("auth_manually_logged_out");
    return value === "true";
  } catch (e) {
    console.error("Error reading from localStorage:", e);
    return false;
  }
};

const setManuallyLoggedOutInStorage = (value: boolean): void => {
  try {
    localStorage.setItem("auth_manually_logged_out", value.toString());
  } catch (e) {
    console.error("Error writing to localStorage:", e);
  }
};

// Define initial state
const initialState: AuthState = {
  user: null,
  isAuthenticated: false,
  isLoading: false,
  error: null,
  manuallyLoggedOut: getManuallyLoggedOutFromStorage(),
};

// Create store
export const $auth = createStore<AuthState>(initialState)
  .on(checkAuthFx.pending, (state) => {
    console.log("checkAuthFx.pending handler called", { prevState: state });
    return {
      // Add pending handler
      ...state,
      isLoading: true,
      error: null,
    };
  })
  .on(checkAuthFx.doneData, (state, result) => {
    console.log("checkAuthFx.doneData handler called", {
      result,
      prevState: state,
    });

    // If a user is found, reset the manuallyLoggedOut flag in localStorage
    if (result) {
      setManuallyLoggedOutInStorage(false);
      console.log(
        "checkAuthFx.doneData: User found, reset manuallyLoggedOut to false in localStorage",
      );
    } else {
      console.log(
        "checkAuthFx.doneData: No user found, keeping manuallyLoggedOut as",
        state.manuallyLoggedOut,
      );
    }

    return {
      ...state,
      isLoading: false, // Set loading false on done
      user: result
        ? {
            id: result.profile.sub,
            name:
              result.profile.name || result.profile.preferred_username || "",
            email: result.profile.email || "",
            roles: result.profile.roles || [],
          }
        : null,
      isAuthenticated: !!result,
      // If a user is found, reset the manuallyLoggedOut flag
      manuallyLoggedOut: result ? false : state.manuallyLoggedOut,
    };
  })
  .on(checkAuthFx.fail, (state, { error }) => {
    console.log("checkAuthFx.fail handler called", { error, prevState: state });
    return {
      // Add fail handler
      ...state,
      isLoading: false,
      error: error.message,
    };
  })
  .on(handleCallbackFx.pending, (state) => {
    console.log("handleCallbackFx.pending handler called", {
      prevState: state,
    });
    return {
      // Add pending handler for callback
      ...state,
      isLoading: true,
      error: null,
    };
  })
  .on(handleCallbackFx.doneData, (state, result) => {
    console.log("handleCallbackFx.doneData handler called", {
      result,
      prevState: state,
    });

    // Reset flag in localStorage when user successfully logs in
    setManuallyLoggedOutInStorage(false);
    console.log(
      "handleCallbackFx.doneData: Reset manuallyLoggedOut to false in localStorage",
    );

    return {
      ...state,
      isLoading: false, // Set loading false on done
      user: {
        id: result.profile.sub,
        name: result.profile.name || result.profile.preferred_username || "",
        email: result.profile.email || "",
        roles: result.profile.roles || [],
      },
      isAuthenticated: true,
      manuallyLoggedOut: false, // Reset the flag when user successfully logs in
    };
  })
  .on(handleCallbackFx.fail, (state, { error }) => {
    console.log("handleCallbackFx.fail handler called", {
      error,
      prevState: state,
    });
    return {
      // Add fail handler for callback
      ...state,
      isLoading: false,
      error: error.message,
    };
  })
  .on(setUser, (state, user) => ({
    ...state,
    user,
    isAuthenticated: !!user,
  }))
  .on(setIsLoading, (state, isLoading) => ({
    ...state,
    isLoading,
  }))
  .on(setError, (state, error) => ({
    ...state,
    error,
  }))
  .on(loginFx.pending, (state) => ({
    ...state,
    isLoading: true,
    error: null,
  }))
  .on(loginFx.done, (state) => ({
    ...state,
    isLoading: false,
    error: null,
  }))
  .on(loginFx.fail, (state, { error }) => ({
    ...state,
    isLoading: false,
    error: error.message,
  }))
  .on(logoutFx.pending, (state) => ({
    ...state,
    isLoading: true,
  }))
  .on(logoutFx.done, () => {
    // Set flag in localStorage
    setManuallyLoggedOutInStorage(true);
    console.log("logoutFx.done: Set manuallyLoggedOut to true in localStorage");

    return {
      ...initialState,
      manuallyLoggedOut: true, // Set flag when user manually logs out
    };
  })
  .on(logoutFx.fail, (state, { error }) => ({
    ...state,
    isLoading: false,
    error: error.message,
  }))
  .on(logout, () => {
    // Set flag in localStorage
    setManuallyLoggedOutInStorage(true);
    console.log("logout event: Set manuallyLoggedOut to true in localStorage");

    return {
      ...initialState,
      manuallyLoggedOut: true, // Set flag when user manually logs out
    };
  });
