import {
  UserManager,
  WebStorageStateStore,
  UserManagerSettings,
} from "oidc-client-ts";
import { OidcService } from "@merchant-api/services/OidcService";
import { oidcConnectResponseDto } from "@merchant-api/models/oidcConnectResponseDto";
import { OpenAPI } from "@merchant-api/core/OpenAPI";
import { createHashUrl, normalizeUrl } from "@shared/lib/url-utils";
import { getApiUrlSingleton } from "@shared/lib/api-url";

let userManagerInstance: UserManager | null = null;

export const getOidcConfig = async (): Promise<UserManagerSettings> => {
  try {
    // Get the API URL using our utility function
    try {
      // This will handle auto-detection if needed
      const apiUrl = getApiUrlSingleton();

      console.log(`Using API URL for OIDC config: ${apiUrl}`);
      // Ensure OpenAPI.BASE is set before making any API calls
      // This is a safeguard in case the initialize.ts module hasn't run yet
      OpenAPI.BASE = apiUrl;
    } catch (apiUrlError) {
      console.error("FATAL ERROR: Failed to get API URL:", apiUrlError);
      throw apiUrlError;
    }

    console.log("Fetching OIDC config using OidcService");

    // Use the OidcService to fetch the OIDC configuration
    console.log("About to call OidcService.getOidcJson()");
    let config: oidcConnectResponseDto;
    try {
      config = await OidcService.getOidcJson();
      console.log("OIDC config received:", config);
    } catch (fetchError) {
      console.error("Error from OidcService.getOidcJson():", fetchError);
      console.error(
        "This may be due to CLIENT_API_URL being incorrect or the API server being unavailable",
      );
      console.error(`Current OpenAPI.BASE: ${OpenAPI.BASE}`);
      throw fetchError;
    }

    if (!config || !config.authorityUrl || !config.clientId) {
      console.error(
        "FATAL ERROR: Invalid OIDC config received from API - missing authorityUrl or clientId",
      );
      throw new Error("Invalid OIDC config received from API");
    }

    // Log the authorityUrl that will be used for OIDC
    console.log(`Using OIDC authorityUrl: ${config.authorityUrl}`);

    // Important: We use the authorityUrl directly from the OIDC config
    // This URL will be used for all identity/.well-known endpoint checks

    // Properly construct the redirect URI using our utility functions
    const redirectUri = createHashUrl(
      window.location.origin,
      window.appConfig?.basePath,
      "auth/callback",
    );

    // Create the post-logout redirect URI
    const logoutRedirectUri = createHashUrl(
      window.location.origin,
      window.appConfig?.basePath,
      "",
    );

    // Normalize the authority URL to ensure it has a trailing slash
    var normalizedAuthorityUrl = normalizeUrl(config.authorityUrl);

    console.log(
      `Using normalized OIDC authorityUrl: ${normalizedAuthorityUrl}`,
    );
    console.log(`Constructed redirect URI: ${redirectUri}`);
    console.log(`Constructed logout redirect URI: ${logoutRedirectUri}`);

    return {
      authority: normalizedAuthorityUrl, // Using normalized authorityUrl
      client_id: config.clientId,
      redirect_uri: redirectUri,
      response_type: "code",
      scope: "openid profile email offline_access", // Added offline_access for refresh tokens
      post_logout_redirect_uri: logoutRedirectUri,
      userStore: new WebStorageStateStore({ store: window.localStorage }),
      loadUserInfo: true,
      automaticSilentRenew: true,
      revokeTokensOnSignout: true,
      monitorSession: true,
    };
  } catch (error) {
    console.error(
      "FATAL ERROR: Failed to initialize OIDC configuration:",
      error,
    );
    throw error; // Re-throw to propagate the error
  }
};

export const initializeUserManager = async (): Promise<UserManager> => {
  if (userManagerInstance) {
    return userManagerInstance;
  }
  const settings = await getOidcConfig();
  userManagerInstance = new UserManager(settings);
  return userManagerInstance;
};

// Function to get the initialized instance, throws error if not initialized
export const getUserManager = (): UserManager => {
  if (!userManagerInstance) {
    throw new Error(
      "UserManager not initialized. Call initializeUserManager first.",
    );
  }
  return userManagerInstance;
};
