import React, { useEffect, useState } from "react";
import { useNavigate } from "react-router-dom";
import { handleCallbackFx, setIsLoading } from "../model"; // Import the effect and events
import { useAuth } from "../view-model";
import { cleanupUrl } from "@shared/lib/url-utils";

export const AuthCallback: React.FC = () => {
  const navigate = useNavigate();
  const { isAuthenticated } = useAuth();

  const [status, setStatus] = useState("processing");

  // We're using the imported cleanupUrl function from url-utils

  useEffect(() => {
    const processCallback = async () => {
      // If user is already authenticated, just redirect to home
      if (isAuthenticated) {
        console.log(
          "AuthCallback: User is already authenticated, redirecting to home",
        );
        setStatus("success");
        setIsLoading(false);

        // Redirect to home page
        setTimeout(() => {
          navigate("/", { replace: true });
        }, 500);
        return;
      }

      try {
        console.log("AuthCallback: Starting to process callback");
        // Use the effect to handle the callback and update the store
        await handleCallbackFx();
        console.log("AuthCallback: Callback processed successfully");

        // Explicitly set loading to false to prevent loading screen
        setIsLoading(false);
        setStatus("success");

        // Redirect to the intended page after successful login
        // Retrieve the stored return URL or default to '/'
        const returnUrl = sessionStorage.getItem("oidc_return_url") || "/";
        sessionStorage.removeItem("oidc_return_url"); // Clean up

        console.log("AuthCallback: Redirecting to", returnUrl);

        // Clean up URL
        cleanupUrl();

        // Add a small delay before redirecting to ensure state is updated
        setTimeout(() => {
          navigate(returnUrl, { replace: true });
        }, 500);
      } catch (err) {
        console.error("Error handling OIDC callback:", err);

        // If the error is about missing state but user is authenticated,
        // we can just redirect to home
        const errorMessage = err instanceof Error ? err.message : String(err);
        if (
          errorMessage.includes("No matching state found") &&
          isAuthenticated
        ) {
          console.log(
            "AuthCallback: State error but user is authenticated, redirecting to home",
          );
          setStatus("success");
          setIsLoading(false);

          // Clean up URL
          cleanupUrl();

          setTimeout(() => {
            navigate("/", { replace: true });
          }, 500);
          return;
        }

        setStatus("error");
        setIsLoading(false);

        // Clean up URL
        cleanupUrl();

        // Redirect to login page on error after a small delay
        setTimeout(() => {
          navigate("/login", { replace: true });
        }, 500);
      }
    };

    processCallback();
  }, [navigate, isAuthenticated]);

  // Display a loading indicator while processing the callback using the same design as login page
  return (
    <div className="flex items-center justify-center min-h-screen bg-[#131215]">
      <div className="w-full max-w-md p-8 space-y-8 bg-[#26242a] rounded-lg shadow-lg">
        <div className="text-center">
          <h1 className="text-2xl font-bold text-white">App</h1>

          {status === "processing" && (
            <div className="mt-6">
              <div className="flex flex-col items-center justify-center">
                <div className="animate-spin rounded-full h-12 w-12 border-t-2 border-b-2 border-[#653cee] mb-4"></div>
                <p className="text-white text-lg">Completing login...</p>
                <p className="text-gray-400 text-sm mt-2">
                  Please wait while we verify your credentials
                </p>
              </div>
            </div>
          )}

          {status === "success" && (
            <div className="mt-6">
              <div className="flex flex-col items-center justify-center">
                <div className="text-green-500 text-5xl mb-4">✓</div>
                <p className="text-white text-lg">Login successful!</p>
                <p className="text-gray-400 text-sm mt-2">
                  Redirecting to your dashboard...
                </p>
              </div>
            </div>
          )}

          {status === "error" && (
            <div className="mt-6">
              <div className="flex flex-col items-center justify-center">
                <div className="text-red-500 text-5xl mb-4">✗</div>
                <p className="text-white text-lg">Login failed</p>
                <p className="text-gray-400 text-sm mt-2">
                  Redirecting to login page...
                </p>
              </div>
            </div>
          )}
        </div>
      </div>
    </div>
  );
};
