import React, { ReactNode, useEffect, useState } from "react";
import { useAuth } from "../view-model";
import { checkAuthFx, setError, setIsLoading } from "../model"; // Import effect and events
import { initializeUserManager } from "../oidc-config"; // Import initializer
import { checkApiUrlConfig } from "@shared/lib/diagnostics"; // Import diagnostics utility

interface AuthProviderProps {
  children: ReactNode;
}

export const AuthProvider: React.FC<AuthProviderProps> = ({ children }) => {
  const { isLoading, error } = useAuth();
  const [isInitialized, setIsInitialized] = useState(false);

  // Initialize UserManager and check authentication status on component mount
  useEffect(() => {
    let isMounted = true; // Flag to prevent state updates after unmount

    const initAuth = async () => {
      try {
        console.log("AuthProvider: Starting initialization");

        // Run diagnostics to check API URL configuration
        console.log("AuthProvider: Running API URL diagnostics");
        const apiUrl = checkApiUrlConfig();
        if (!apiUrl) {
          console.error(
            "AuthProvider: window.appConfig.apiUrl is not defined, authentication will likely fail",
          );
        }

        await initializeUserManager(); // Initialize first
        console.log("AuthProvider: UserManager initialized");

        if (isMounted) {
          const result = await checkAuthFx(); // Then check auth status
          console.log(
            "AuthProvider: Authentication check completed",
            result ? "User authenticated" : "No user",
          );

          if (isMounted) {
            setIsInitialized(true);
            setIsLoading(false); // Explicitly set loading to false
            console.log(
              "AuthProvider: Initialization complete, isInitialized set to true, isLoading set to false",
            );
          }
        }
      } catch (initError) {
        console.error(
          "FATAL ERROR: Failed to initialize OIDC or check auth:",
          initError,
        );

        if (isMounted) {
          setError(
            initError instanceof Error
              ? initError.message
              : "Authentication initialization failed",
          );
          setIsInitialized(true); // Mark as initialized even on error to stop loading screen
          setIsLoading(false); // Explicitly set loading to false
          console.log(
            "AuthProvider: Initialization failed, but isInitialized set to true, isLoading set to false",
          );
        }
      }
    };

    initAuth();

    // Cleanup function to prevent state updates after unmount
    return () => {
      isMounted = false;
    };
  }, []);

  // If not initialized yet, render nothing (or a minimal loader)
  if (!isInitialized) {
    console.log("AuthProvider render: Not initialized yet", {
      isLoading,
      isInitialized,
    });
    return null; // Return nothing, the login page will handle the loading state
  }

  // Show error state
  if (error) {
    console.log("AuthProvider render: Showing error state", { error });
    return (
      <div className="flex items-center justify-center min-h-screen bg-[#131215]">
        <div className="w-full max-w-md p-8 space-y-8 bg-[#26242a] rounded-lg shadow-lg">
          <div className="text-center">
            <h1 className="text-2xl font-bold text-white">App</h1>

            <div className="mt-6">
              <div className="flex flex-col items-center justify-center">
                <div className="text-red-500 text-5xl mb-4">✗</div>
                <p className="text-white text-lg">Authentication Error</p>
                <p className="text-red-400 text-sm mt-2 mb-4">{error}</p>

                <div className="text-white bg-[#1a191c] p-4 rounded text-left mb-6 overflow-auto max-h-60 w-full">
                  <p className="mb-2 font-semibold">Possible solutions:</p>
                  <ol className="list-decimal pl-5 space-y-2 text-sm text-gray-300">
                    <li>
                      Check that the API server is running at the URL specified
                      in window.appConfig.apiUrl
                    </li>
                    <li>
                      Verify that the /oidc.json endpoint is accessible from the
                      API
                    </li>
                    <li>
                      Ensure the OIDC configuration has valid authorityUrl and
                      clientId values
                    </li>
                    <li>Check browser console for detailed error messages</li>
                    <li>
                      If window.appConfig.apiUrl is not defined, check your
                      environment variables and restart the development server
                    </li>
                  </ol>
                </div>

                <button
                  onClick={() => window.location.reload()}
                  className="w-full px-4 py-2 text-white bg-[#653cee] rounded-md hover:bg-[#4e2eb8] focus:outline-none focus:ring-2 focus:ring-[#653cee] focus:ring-offset-2 focus:ring-offset-[#26242a] transition-colors"
                >
                  Retry
                </button>
              </div>
            </div>
          </div>
        </div>
      </div>
    );
  }

  // Render children when authentication is ready
  console.log(
    "AuthProvider render: Rendering children, authentication is ready",
  );
  return <>{children}</>;
};
