import React, { useEffect } from "react";
import { useAuth } from "../view-model";

export const LoginPage: React.FC = () => {
  const { login, isLoading, manuallyLoggedOut, isAuthenticated, user } =
    useAuth();

  // Auto-login when component mounts if not manually logged out and not already authenticated
  useEffect(() => {
    // If already authenticated, no need to do anything
    if (isAuthenticated || user) {
      console.log(
        "LoginPage: User is already authenticated, no need to auto-login",
      );
      return;
    }

    // Check localStorage directly for the manuallyLoggedOut flag
    const isManuallyLoggedOut =
      localStorage.getItem("auth_manually_logged_out") === "true";
    console.log(
      "LoginPage: localStorage auth_manually_logged_out =",
      isManuallyLoggedOut,
    );
    console.log("LoginPage: manuallyLoggedOut from state =", manuallyLoggedOut);

    // Only auto-login if not manually logged out (checking both state and localStorage)
    // and not already loading
    if (!isManuallyLoggedOut && !manuallyLoggedOut && !isLoading) {
      console.log(
        "LoginPage: Auto-authenticating because user has not manually logged out",
      );
      login().catch((err) => {
        console.error("FATAL ERROR: Auto-login failed:", err);
      });
    } else {
      console.log(
        "LoginPage: Not auto-authenticating because user manually logged out or is loading",
      );
      console.log(
        "isManuallyLoggedOut:",
        isManuallyLoggedOut,
        "manuallyLoggedOut:",
        manuallyLoggedOut,
        "isLoading:",
        isLoading,
      );
    }
  }, [login, manuallyLoggedOut, isLoading, isAuthenticated, user]);

  const handleLogin = async () => {
    try {
      console.log("LoginPage: Manual login initiated");

      // Reset the manuallyLoggedOut flag in localStorage when user manually clicks login
      localStorage.setItem("auth_manually_logged_out", "false");
      console.log(
        "LoginPage: Reset manuallyLoggedOut to false in localStorage",
      );

      // Store the current path as the return URL after login
      const returnUrl = window.location.hash.substring(1) || "/";
      sessionStorage.setItem("oidc_return_url", returnUrl);
      console.log("LoginPage: Stored return URL:", returnUrl);

      await login();
    } catch (err) {
      console.error("FATAL ERROR: Login failed:", err);
      alert(
        `Login failed: ${err instanceof Error ? err.message : "Unknown error"}.\n\nCheck the console for more details.`,
      );
    }
  };

  return (
    <div className="flex items-center justify-center min-h-screen bg-[#131215]">
      <div className="w-full max-w-md p-8 space-y-8 bg-[#26242a] rounded-lg shadow-lg">
        <div className="text-center">
          <h1 className="text-2xl font-bold text-white">App</h1>
          <p className="mt-2 text-gray-400">
            Sign in with your identity provider
          </p>
        </div>

        <div className="mt-8">
          <button
            onClick={handleLogin}
            disabled={isLoading}
            className="w-full px-4 py-2 text-white bg-[#653cee] rounded-md hover:bg-[#4e2eb8] focus:outline-none focus:ring-2 focus:ring-[#653cee] focus:ring-offset-2 focus:ring-offset-[#26242a] disabled:opacity-50 disabled:cursor-not-allowed flex items-center justify-center"
          >
            {isLoading ? (
              <>
                <div className="animate-spin rounded-full h-5 w-5 border-t-2 border-b-2 border-white mr-2"></div>
                <span>Authenticating...</span>
              </>
            ) : (
              "Sign in"
            )}
          </button>

          <div className="mt-6 text-sm text-gray-400 text-center">
            <p>This will redirect you to your configured identity provider.</p>
            <p className="mt-2">
              Make sure your OIDC configuration is properly set up.
            </p>
          </div>
        </div>
      </div>
    </div>
  );
};
