import React, { ReactNode } from "react";
import { useAuth } from "../view-model";
import { Navigate, useLocation } from "react-router-dom";

interface ProtectedRouteProps {
  children: ReactNode;
  requiredRoles?: string[];
  redirectTo?: string;
}

export const ProtectedRoute: React.FC<ProtectedRouteProps> = ({
  children,
  requiredRoles = [],
  redirectTo = "/login",
}) => {
  const { isAuthenticated, isLoading, hasRole } = useAuth();
  const location = useLocation();

  console.log("ProtectedRoute: Rendering with state", {
    isAuthenticated,
    isLoading,
    path: location.pathname,
  });

  // If still loading, redirect to login page which will show the loading state
  if (isLoading) {
    console.log("ProtectedRoute: Still loading, redirecting to login page");
    return <Navigate to={redirectTo} replace />;
  }

  // Redirect to login if not authenticated
  if (!isAuthenticated) {
    console.log(
      `User not authenticated, redirecting to ${redirectTo} from ${location.pathname}`,
    );
    // Store the current location so we can redirect back after login
    sessionStorage.setItem("oidc_return_url", location.pathname);
    return (
      <Navigate to={redirectTo} state={{ from: location.pathname }} replace />
    );
  }

  // Check for required roles if specified
  if (requiredRoles.length > 0) {
    const hasRequiredRole = requiredRoles.some((role) => hasRole(role));

    if (!hasRequiredRole) {
      return (
        <div className="flex items-center justify-center h-screen">
          <div className="text-center text-red-500">
            <div className="text-xl mb-4">Access Denied</div>
            <div>You don't have permission to access this page.</div>
          </div>
        </div>
      );
    }
  }

  // Render the protected content
  console.log("ProtectedRoute: Rendering protected content");
  return <>{children}</>;
};
