import { useAuth } from "@domains/auth";
import { ApiRequestOptions, ApiResponse, ApiService } from "./types";

// Custom hook for making authenticated API requests
export const useApiService = (): ApiService => {
  const { user, isAuthenticated } = useAuth();

  // Note: This is a simplified version that doesn't handle tokens
  // In a real application, you would need to implement proper token handling

  // Base fetch function with authentication
  const fetchWithAuth = async <T = any>(
    url: string,
    options: ApiRequestOptions = {},
  ): Promise<ApiResponse<T>> => {
    // No token expiration check or refresh in this simplified version

    // Set up headers with authentication
    const headers: Record<string, string> = {
      ...options.headers,
      "Content-Type": "application/json",
    };

    // In a real application, you would add the authorization header here
    // For now, we're just checking if the user is authenticated
    if (!isAuthenticated) {
      console.warn(
        "User is not authenticated. Request may fail if authentication is required.",
      );
    }

    // Make the request
    const response = (await fetch(url, {
      ...options,
      headers,
    })) as ApiResponse<T>;

    // Handle unauthorized errors
    if (response.status === 401) {
      console.error("Authentication error: Unauthorized request");
      // In a real application, you might want to redirect to login page
      // or trigger a login flow here
    }

    return response;
  };

  // Helper methods for common HTTP methods
  const get = <T = any>(
    url: string,
    options: ApiRequestOptions = {},
  ): Promise<ApiResponse<T>> =>
    fetchWithAuth<T>(url, { ...options, method: "GET" });

  const post = <T = any>(
    url: string,
    data?: any,
    options: ApiRequestOptions = {},
  ): Promise<ApiResponse<T>> =>
    fetchWithAuth<T>(url, {
      ...options,
      method: "POST",
      body: JSON.stringify(data),
    });

  const put = <T = any>(
    url: string,
    data?: any,
    options: ApiRequestOptions = {},
  ): Promise<ApiResponse<T>> =>
    fetchWithAuth<T>(url, {
      ...options,
      method: "PUT",
      body: JSON.stringify(data),
    });

  const del = <T = any>(
    url: string,
    options: ApiRequestOptions = {},
  ): Promise<ApiResponse<T>> =>
    fetchWithAuth<T>(url, { ...options, method: "DELETE" });

  return {
    fetchWithAuth,
    get,
    post,
    put,
    delete: del,
  };
};
